function XrkAccessExample(sFilename)

%% open library
%% loadlibrary(fullfile(pwd, 'MatLabXRK-2017-32-ReleaseU.dll'), @AccessAimXrk, 'alias', 'AccessAimXrk');
%% loadlibrary(fullfile(pwd, 'MatLabXRK-2017-64-ReleaseU.dll'), @AccessAimXrk, 'alias', 'AccessAimXrk');
%% loadlibrary(fullfile(pwd, 'MatLabXRK-2019-64-ReleaseU.dll'), @AccessAimXrk, 'alias', 'AccessAimXrk');
loadlibrary(fullfile(pwd, 'MatLabXRK-2022-64-ReleaseU.dll'), @AccessAimXrk, 'alias', 'AccessAimXrk');
calllib('AccessAimXrk', 'get_library_date')
calllib('AccessAimXrk', 'get_library_time')

%% open XRK / XRZ file
if (nargin < 1) || (exist(sFilename, 'file') ~= 2)
    [sFile, sPath] = uigetfile({'*.xrk;*.xrz', 'AiM data files'}, 'Select AiM data file', pwd);
    if ischar(sFile)
        sFilename = fullfile(sPath, sFile);
    end
end
assert(exist(sFilename, 'file') == 2, 'file ''%s'' does not exist', sFilename);
iFile = calllib('AccessAimXrk', 'open_file', sFilename);
assert(iFile > 0, 'invalid file index');

%% get some file information
fprintf('vehicle name:\t%s\n', calllib('AccessAimXrk', 'get_vehicle_name', iFile));
fprintf('track name:\t\t%s\n', calllib('AccessAimXrk', 'get_track_name', iFile));
fprintf('racer name:\t\t%s\n', calllib('AccessAimXrk', 'get_racer_name', iFile));
fprintf('championship:\t%s\n', calllib('AccessAimXrk', 'get_championship_name', iFile));
fprintf('session type:\t\t%s\n', calllib('AccessAimXrk', 'get_session_type_name', iFile));
oTm = calllib('AccessAimXrk', 'get_date_and_time', iFile);
fprintf('timestamp:\t\t%04u-%02u-%02u %02u:%02u:%02u\n', oTm.Value.tm_year + 1900, oTm.Value.tm_mon + 1, ...
    oTm.Value.tm_mday, oTm.Value.tm_hour, oTm.Value.tm_min, oTm.Value.tm_sec);

%% get number of laps and number of channels
iLapCount = calllib('AccessAimXrk', 'get_laps_count', iFile) %#ok
iChannelCount = calllib('AccessAimXrk', 'get_channels_count', iFile) %#ok

%% get lap information
pStart = libpointer('doublePtr', nan);
pDuration = libpointer('doublePtr', nan);
aLapTimes = nan(iLapCount, 2);
for i=1:iLapCount
    calllib('AccessAimXrk', 'get_lap_info', iFile, i-1, pStart, pDuration);
    aLapTimes(i, :) = [pStart.Value, pDuration.Value];
end
aLapTimes %#ok

%% user has to select lap
[iLap, bValid] = listdlg('PromptString', 'Select a lap:', 'SelectionMode', 'single', ...
                'ListString', arrayfun(@num2str, 1:iLapCount, 'UniformOutput', false));
if ~bValid
    iLap = str2double(sSelection);
end

%% get channel names / units
cChannelNames = cell(1, iChannelCount);
cChannelUnits = cell(1, iChannelCount);
for i=1:iChannelCount
    cChannelNames{i} = calllib('AccessAimXrk', 'get_channel_name_no_spaces', iFile, i-1);
    cChannelUnits{i} = calllib('AccessAimXrk', 'get_channel_units', iFile, i-1);
end

%% user has to select channel
[iChannel, bValid] = listdlg('PromptString', 'Select a channel:', 'SelectionMode', 'single', ...
                'ListString', cChannelNames);
if ~bValid
    iChannel = 1;
end

%% get channel samples
iSamplesCount = calllib('AccessAimXrk', 'get_lap_channel_samples_count', iFile, iLap-1, iChannel-1);
if iSamplesCount == 0
    fprintf('no samples for channel ''%s''\n', cChannelNames{iChannel});
else
    pTime = libpointer('doublePtr', nan(1, iSamplesCount));
    pData = libpointer('doublePtr', nan(1, iSamplesCount));
    calllib('AccessAimXrk', 'get_lap_channel_samples', iFile, iLap-1, iChannel-1, pTime, pData, ...
        iSamplesCount);
    figure;
    plot(pTime.Value, pData.Value);
    title(sprintf('%s [%s]', cChannelNames{iChannel}, cChannelUnits{iChannel}));
end

%% get GPS channel names
iGpsChannelCount = calllib('AccessAimXrk', 'get_GPS_channels_count', iFile) %#ok
cGpsChannelNames = cell(1, iGpsChannelCount);
cGpsUnits = cell(1, iGpsChannelCount);
for i=1:iGpsChannelCount
    cGpsChannelNames{i} = calllib('AccessAimXrk', 'get_GPS_channel_name_no_spaces', iFile, i-1);
    cGpsUnits{i} = calllib('AccessAimXrk', 'get_GPS_channel_units', iFile, i-1);
end

%% user has to select channel
[iChannel, bValid] = listdlg('PromptString', 'Select a GPS channel:', 'SelectionMode', 'single', ...
                'ListString', cGpsChannelNames);
if ~bValid
    iChannel = 1;
end

%% get GPS channel samples
iSamplesCount = calllib('AccessAimXrk', 'get_lap_GPS_channel_samples_count', iFile, iLap-1, iChannel-1);
if iSamplesCount == 0
    fprintf('no samples for GPS channel ''%s''\n', cGpsChannelNames{iChannel});
else
    pTime = libpointer('doublePtr', nan(1, iSamplesCount));
    pData = libpointer('doublePtr', nan(1, iSamplesCount));
    calllib('AccessAimXrk', 'get_lap_GPS_channel_samples', iFile, iLap-1, iChannel-1, pTime, pData, ...
        iSamplesCount);
    figure;
    plot(pTime.Value, pData.Value);
    title(sprintf('%s [%s]', cGpsChannelNames{iChannel}, cGpsUnits{iChannel}));
end

%% get raw GPS channel names
iGpsRawChannelCount = calllib('AccessAimXrk', 'get_GPS_raw_channels_count', iFile) %#ok
cGpsRawChannelNames = cell(1, iGpsRawChannelCount);
cGpsRawUnits = cell(1, iGpsRawChannelCount);
for i=1:iGpsRawChannelCount
    cGpsRawChannelNames{i} = calllib('AccessAimXrk', 'get_GPS_raw_channel_name_no_spaces', iFile, i-1);
    cGpsRawUnits{i} = calllib('AccessAimXrk', 'get_GPS_raw_channel_units', iFile, i-1);
end
cGpsRawChannelNames %#ok
cGpsRawUnits %#ok

%% user has to select channel
[iChannel, bValid] = listdlg('PromptString', 'Select a GPS raw channel:', 'SelectionMode', 'single', ...
                'ListString', cGpsRawChannelNames);
if ~bValid
    iChannel = 1;
end

%% get raw GPS channel samples
iSamplesCount = calllib('AccessAimXrk', 'get_lap_GPS_raw_channel_samples_count', iFile, iLap-1, iChannel-1);
if iSamplesCount == 0
    fprintf('no samples for GPS raw channel ''%s''\n', cGpsRawChannelNames{iChannel});
else
    pTime = libpointer('doublePtr', nan(1, iSamplesCount));
    pData = libpointer('doublePtr', nan(1, iSamplesCount));
    calllib('AccessAimXrk', 'get_lap_GPS_raw_channel_samples', iFile, iLap-1, iChannel-1, pTime, pData, ...
        iSamplesCount);
    figure;
    plot(pTime.Value, pData.Value);
    title(sprintf('%s [%s]', cGpsRawChannelNames{iChannel}, cGpsRawUnits{iChannel}));
end

%% close file
calllib('AccessAimXrk', 'close_file_i', iFile)
